"""write GUI tables.

- write_table_txt : as text file
- write_table_csv : as csv file
"""

__all__ = ['write_table_txt', 'write_table_csv']


def write_table_txt():
    """Write a table in a text (txt) file."""
    pass


def write_table_csv():
    """Write a table in a CSV (csv) file."""
    pass
