# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/etienne/Toolbox/visbrain/visbrain/visuals/cbar/gui/CbarPyQtGui.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(489, 881)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self._cbar_grp = QtWidgets.QGroupBox(Form)
        self._cbar_grp.setCheckable(True)
        self._cbar_grp.setChecked(False)
        self._cbar_grp.setObjectName("_cbar_grp")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self._cbar_grp)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.CbarLayout = QtWidgets.QVBoxLayout()
        self.CbarLayout.setContentsMargins(-1, 0, -1, -1)
        self.CbarLayout.setObjectName("CbarLayout")
        self.groupBox_4 = QtWidgets.QGroupBox(self._cbar_grp)
        self.groupBox_4.setObjectName("groupBox_4")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.gridLayout_9 = QtWidgets.QGridLayout()
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.object = QtWidgets.QComboBox(self.groupBox_4)
        self.object.setObjectName("object")
        self.gridLayout_9.addWidget(self.object, 0, 2, 1, 1)
        self.line_12 = QtWidgets.QFrame(self.groupBox_4)
        self.line_12.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_12.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_12.setObjectName("line_12")
        self.gridLayout_9.addWidget(self.line_12, 0, 1, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.groupBox_4)
        self.label_18.setObjectName("label_18")
        self.gridLayout_9.addWidget(self.label_18, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_9.addItem(spacerItem, 1, 2, 1, 1)
        self.verticalLayout_7.addLayout(self.gridLayout_9)
        self.CbarLayout.addWidget(self.groupBox_4)
        self.groupBox_3 = QtWidgets.QGroupBox(self._cbar_grp)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.gridLayout_7 = QtWidgets.QGridLayout()
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.border = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.border.setFont(font)
        self.border.setObjectName("border")
        self.gridLayout_7.addWidget(self.border, 4, 0, 1, 1)
        self.line_7 = QtWidgets.QFrame(self.groupBox_3)
        self.line_7.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_7.setObjectName("line_7")
        self.gridLayout_7.addWidget(self.line_7, 1, 1, 1, 1)
        self.bw = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.bw.setDecimals(1)
        self.bw.setMinimum(0.1)
        self.bw.setObjectName("bw")
        self.gridLayout_7.addWidget(self.bw, 4, 2, 1, 1)
        self.width = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.width.setSingleStep(0.01)
        self.width.setObjectName("width")
        self.gridLayout_7.addWidget(self.width, 3, 2, 1, 1)
        self.txtCol = QtWidgets.QLineEdit(self.groupBox_3)
        self.txtCol.setObjectName("txtCol")
        self.gridLayout_7.addWidget(self.txtCol, 1, 2, 1, 1)
        self.ndigits = QtWidgets.QSpinBox(self.groupBox_3)
        self.ndigits.setMinimum(1)
        self.ndigits.setMaximum(10)
        self.ndigits.setProperty("value", 1)
        self.ndigits.setObjectName("ndigits")
        self.gridLayout_7.addWidget(self.ndigits, 2, 2, 1, 1)
        self.line_9 = QtWidgets.QFrame(self.groupBox_3)
        self.line_9.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_9.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_9.setObjectName("line_9")
        self.gridLayout_7.addWidget(self.line_9, 2, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.gridLayout_7.addWidget(self.label_9, 1, 0, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_16.setFont(font)
        self.label_16.setObjectName("label_16")
        self.gridLayout_7.addWidget(self.label_16, 2, 0, 1, 1)
        self.line_10 = QtWidgets.QFrame(self.groupBox_3)
        self.line_10.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_10.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_10.setObjectName("line_10")
        self.gridLayout_7.addWidget(self.line_10, 3, 1, 1, 1)
        self.bckCol = QtWidgets.QLineEdit(self.groupBox_3)
        self.bckCol.setObjectName("bckCol")
        self.gridLayout_7.addWidget(self.bckCol, 0, 2, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.gridLayout_7.addWidget(self.label_17, 3, 0, 1, 1)
        self.line_11 = QtWidgets.QFrame(self.groupBox_3)
        self.line_11.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_11.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_11.setObjectName("line_11")
        self.gridLayout_7.addWidget(self.line_11, 4, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.gridLayout_7.addWidget(self.label_8, 0, 0, 1, 1)
        self.line_6 = QtWidgets.QFrame(self.groupBox_3)
        self.line_6.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.gridLayout_7.addWidget(self.line_6, 0, 1, 1, 1)
        self.verticalLayout_9.addLayout(self.gridLayout_7)
        self.CbarLayout.addWidget(self.groupBox_3)
        self.groupBox_2 = QtWidgets.QGroupBox(self._cbar_grp)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_4.setContentsMargins(0, -1, 0, -1)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.cmapRev = QtWidgets.QCheckBox(self.groupBox_2)
        self.cmapRev.setObjectName("cmapRev")
        self.gridLayout.addWidget(self.cmapRev, 1, 3, 1, 1)
        self.line_2 = QtWidgets.QFrame(self.groupBox_2)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout.addWidget(self.line_2, 4, 1, 1, 1)
        self.vmaxW = QtWidgets.QWidget(self.groupBox_2)
        self.vmaxW.setObjectName("vmaxW")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.vmaxW)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setVerticalSpacing(9)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.over = QtWidgets.QLineEdit(self.vmaxW)
        self.over.setObjectName("over")
        self.gridLayout_3.addWidget(self.over, 1, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.vmaxW)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 0, 0, 1, 1)
        self.vmax = QtWidgets.QDoubleSpinBox(self.vmaxW)
        self.vmax.setObjectName("vmax")
        self.gridLayout_3.addWidget(self.vmax, 0, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.vmaxW)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 1, 0, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout_3)
        self.gridLayout.addWidget(self.vmaxW, 4, 2, 1, 2)
        self.label_2 = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.line_4 = QtWidgets.QFrame(self.groupBox_2)
        self.line_4.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.gridLayout.addWidget(self.line_4, 1, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)
        self.isvmin = QtWidgets.QCheckBox(self.groupBox_2)
        font = QtGui.QFont()
        font.setItalic(True)
        self.isvmin.setFont(font)
        self.isvmin.setObjectName("isvmin")
        self.gridLayout.addWidget(self.isvmin, 3, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 0, 2, 1, 1)
        self.cmap = QtWidgets.QComboBox(self.groupBox_2)
        self.cmap.setObjectName("cmap")
        self.gridLayout.addWidget(self.cmap, 1, 2, 1, 1)
        self.widget = QtWidgets.QWidget(self.groupBox_2)
        self.widget.setObjectName("widget")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.climm = QtWidgets.QDoubleSpinBox(self.widget)
        self.climm.setMinimum(-16777215.0)
        self.climm.setMaximum(16777215.0)
        self.climm.setObjectName("climm")
        self.horizontalLayout_3.addWidget(self.climm)
        self.climM = QtWidgets.QDoubleSpinBox(self.widget)
        self.climM.setMinimum(-16777215.0)
        self.climM.setMaximum(16777215.0)
        self.climM.setProperty("value", 1.0)
        self.climM.setObjectName("climM")
        self.horizontalLayout_3.addWidget(self.climM)
        self.horizontalLayout_4.addLayout(self.horizontalLayout_3)
        self.gridLayout.addWidget(self.widget, 2, 2, 1, 2)
        self.line_3 = QtWidgets.QFrame(self.groupBox_2)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.gridLayout.addWidget(self.line_3, 2, 1, 1, 1)
        self.limTxt = QtWidgets.QCheckBox(self.groupBox_2)
        self.limTxt.setChecked(True)
        self.limTxt.setObjectName("limTxt")
        self.gridLayout.addWidget(self.limTxt, 5, 0, 1, 4)
        self.line = QtWidgets.QFrame(self.groupBox_2)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 3, 1, 1, 1)
        self.vminW = QtWidgets.QWidget(self.groupBox_2)
        self.vminW.setObjectName("vminW")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.vminW)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setVerticalSpacing(9)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_3 = QtWidgets.QLabel(self.vminW)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 0, 0, 1, 1)
        self.vmin = QtWidgets.QDoubleSpinBox(self.vminW)
        self.vmin.setDecimals(3)
        self.vmin.setObjectName("vmin")
        self.gridLayout_2.addWidget(self.vmin, 0, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.vminW)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 1, 0, 1, 1)
        self.under = QtWidgets.QLineEdit(self.vminW)
        self.under.setObjectName("under")
        self.gridLayout_2.addWidget(self.under, 1, 1, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout_2)
        self.gridLayout.addWidget(self.vminW, 3, 2, 1, 2)
        self.isvmax = QtWidgets.QCheckBox(self.groupBox_2)
        font = QtGui.QFont()
        font.setItalic(True)
        self.isvmax.setFont(font)
        self.isvmax.setObjectName("isvmax")
        self.gridLayout.addWidget(self.isvmax, 4, 0, 1, 1)
        self.widget_2 = QtWidgets.QWidget(self.groupBox_2)
        self.widget_2.setMinimumSize(QtCore.QSize(0, 0))
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.autoscale = QtWidgets.QPushButton(self.widget_2)
        self.autoscale.setObjectName("autoscale")
        self.horizontalLayout.addWidget(self.autoscale)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.gridLayout.addWidget(self.widget_2, 6, 0, 1, 4)
        self.verticalLayout_4.addLayout(self.gridLayout)
        self.CbarLayout.addWidget(self.groupBox_2)
        self.groupBox = QtWidgets.QGroupBox(self._cbar_grp)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_5.setContentsMargins(0, -1, 0, -1)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.line_5 = QtWidgets.QFrame(self.groupBox)
        self.line_5.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.gridLayout_4.addWidget(self.line_5, 0, 1, 1, 1)
        self.widget_3 = QtWidgets.QWidget(self.groupBox)
        self.widget_3.setObjectName("widget_3")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.widget_3)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.gridLayout_5 = QtWidgets.QGridLayout()
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.cblabel = QtWidgets.QLineEdit(self.widget_3)
        self.cblabel.setObjectName("cblabel")
        self.gridLayout_5.addWidget(self.cblabel, 0, 1, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.widget_3)
        self.label_11.setObjectName("label_11")
        self.gridLayout_5.addWidget(self.label_11, 1, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.widget_3)
        self.label_10.setObjectName("label_10")
        self.gridLayout_5.addWidget(self.label_10, 0, 0, 1, 1)
        self.cbTxtSz = QtWidgets.QDoubleSpinBox(self.widget_3)
        self.cbTxtSz.setDecimals(1)
        self.cbTxtSz.setObjectName("cbTxtSz")
        self.gridLayout_5.addWidget(self.cbTxtSz, 1, 1, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.widget_3)
        self.label_13.setObjectName("label_13")
        self.gridLayout_5.addWidget(self.label_13, 2, 0, 1, 1)
        self.cbTxtSh = QtWidgets.QDoubleSpinBox(self.widget_3)
        self.cbTxtSh.setDecimals(1)
        self.cbTxtSh.setSingleStep(0.1)
        self.cbTxtSh.setObjectName("cbTxtSh")
        self.gridLayout_5.addWidget(self.cbTxtSh, 2, 1, 1, 1)
        self.verticalLayout_6.addLayout(self.gridLayout_5)
        self.gridLayout_4.addWidget(self.widget_3, 0, 2, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.gridLayout_4.addWidget(self.label_7, 0, 0, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_12.setFont(font)
        self.label_12.setObjectName("label_12")
        self.gridLayout_4.addWidget(self.label_12, 1, 0, 1, 1)
        self.line_8 = QtWidgets.QFrame(self.groupBox)
        self.line_8.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_8.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_8.setObjectName("line_8")
        self.gridLayout_4.addWidget(self.line_8, 1, 1, 1, 1)
        self.widget_4 = QtWidgets.QWidget(self.groupBox)
        self.widget_4.setObjectName("widget_4")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.widget_4)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_8.setSpacing(0)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.gridLayout_6 = QtWidgets.QGridLayout()
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.txtSh = QtWidgets.QDoubleSpinBox(self.widget_4)
        self.txtSh.setDecimals(1)
        self.txtSh.setSingleStep(0.1)
        self.txtSh.setObjectName("txtSh")
        self.gridLayout_6.addWidget(self.txtSh, 1, 1, 1, 1)
        self.txtSz = QtWidgets.QDoubleSpinBox(self.widget_4)
        self.txtSz.setDecimals(1)
        self.txtSz.setObjectName("txtSz")
        self.gridLayout_6.addWidget(self.txtSz, 0, 1, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.widget_4)
        self.label_14.setObjectName("label_14")
        self.gridLayout_6.addWidget(self.label_14, 0, 0, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.widget_4)
        self.label_15.setObjectName("label_15")
        self.gridLayout_6.addWidget(self.label_15, 1, 0, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(40, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_6.addItem(spacerItem4, 2, 1, 1, 1)
        self.verticalLayout_8.addLayout(self.gridLayout_6)
        self.gridLayout_4.addWidget(self.widget_4, 1, 2, 1, 1)
        self.verticalLayout_5.addLayout(self.gridLayout_4)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem5)
        self.CbarLayout.addWidget(self.groupBox)
        self.verticalLayout_10.addLayout(self.CbarLayout)
        self.verticalLayout.addWidget(self._cbar_grp)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self._cbar_grp.setTitle(_translate("Form", "Display"))
        self.groupBox_4.setTitle(_translate("Form", "Objects"))
        self.object.setToolTip(_translate("Form", "<html><head/><body><p>Colorbar object to control</p></body></html>"))
        self.label_18.setText(_translate("Form", "Colorbar of"))
        self.groupBox_3.setTitle(_translate("Form", "Settings"))
        self.border.setToolTip(_translate("Form", "<html><head/><body><p>Display / hide the border of the colorbar</p></body></html>"))
        self.border.setText(_translate("Form", "Border"))
        self.bw.setToolTip(_translate("Form", "<html><head/><body><p>Width of the border</p></body></html>"))
        self.width.setToolTip(_translate("Form", "<html><head/><body><p>Width of the colorbar</p></body></html>"))
        self.txtCol.setToolTip(_translate("Form", "<html><head/><body><p>Color of the text and of the border (if shown)</p></body></html>"))
        self.ndigits.setToolTip(_translate("Form", "<html><head/><body><p>Number of digits a fine control of clim/vmin/vmax</p></body></html>"))
        self.label_9.setText(_translate("Form", "Textcolor"))
        self.label_16.setText(_translate("Form", "Number of\n"
"digits"))
        self.bckCol.setToolTip(_translate("Form", "<html><head/><body><p>Background color of the colorbar canvas</p></body></html>"))
        self.label_17.setText(_translate("Form", "Colorbar\n"
"width"))
        self.label_8.setText(_translate("Form", "Background\n"
"color"))
        self.groupBox_2.setTitle(_translate("Form", "Color properties"))
        self.cmapRev.setToolTip(_translate("Form", "<html><head/><body><p>Reverse the colormap\'s colors</p></body></html>"))
        self.cmapRev.setText(_translate("Form", "Reversed"))
        self.over.setToolTip(_translate("Form", "<html><head/><body><p>Every value under vmax are going to be set to this color</p></body></html>"))
        self.label_5.setText(_translate("Form", "Value"))
        self.vmax.setToolTip(_translate("Form", "<html><head/><body><p>Value of the maximum threshold</p></body></html>"))
        self.label_6.setText(_translate("Form", "Over\n"
"color"))
        self.label_2.setText(_translate("Form", "Clim"))
        self.label.setText(_translate("Form", "Colormap"))
        self.isvmin.setToolTip(_translate("Form", "<html><head/><body><p>Enable / Disable a minimum threshold</p></body></html>"))
        self.isvmin.setText(_translate("Form", "Vmin"))
        self.cmap.setToolTip(_translate("Form", "<html><head/><body><p>Colormap to use</p></body></html>"))
        self.climm.setToolTip(_translate("Form", "<html><head/><body><p>Minimum of the colorbar</p></body></html>"))
        self.climM.setToolTip(_translate("Form", "<html><head/><body><p>Maximum of the colorbar</p></body></html>"))
        self.limTxt.setToolTip(_translate("Form", "<html><head/><body><p>Display vmin/vmax on the colorbar</p></body></html>"))
        self.limTxt.setText(_translate("Form", "Display vmin/vmax text"))
        self.label_3.setText(_translate("Form", "Value"))
        self.vmin.setToolTip(_translate("Form", "<html><head/><body><p>Value of the minimum threshold</p></body></html>"))
        self.label_4.setText(_translate("Form", "Under\n"
"color"))
        self.under.setToolTip(_translate("Form", "<html><head/><body><p>Every values over vmin are going to be set to this color</p></body></html>"))
        self.isvmax.setToolTip(_translate("Form", "<html><head/><body><p>Enable / Disable a maximum threshold</p></body></html>"))
        self.isvmax.setText(_translate("Form", "Vmax"))
        self.autoscale.setToolTip(_translate("Form", "<html><head/><body><p>Automatically scale the colorbar limits to fit with the (minimum, maximum) of currently displayed object</p></body></html>"))
        self.autoscale.setText(_translate("Form", "Autoscale"))
        self.groupBox.setTitle(_translate("Form", "Text properties"))
        self.cblabel.setToolTip(_translate("Form", "<html><head/><body><p>Title of the colorbar</p></body></html>"))
        self.label_11.setText(_translate("Form", "Size"))
        self.label_10.setText(_translate("Form", "Value"))
        self.cbTxtSz.setToolTip(_translate("Form", "<html><head/><body><p>Text size of the colorbar label</p></body></html>"))
        self.label_13.setText(_translate("Form", "Shift"))
        self.cbTxtSh.setToolTip(_translate("Form", "<html><head/><body><p>Shift between the colorbar and the text</p></body></html>"))
        self.label_7.setText(_translate("Form", "Title"))
        self.label_12.setText(_translate("Form", "Limits"))
        self.txtSh.setToolTip(_translate("Form", "<html><head/><body><p>Shift between the colorbar and limits (clim / vmin / vmax)</p></body></html>"))
        self.txtSz.setToolTip(_translate("Form", "<html><head/><body><p>Text size of limits (clim / vmin / vmax)</p></body></html>"))
        self.label_14.setText(_translate("Form", "Size"))
        self.label_15.setText(_translate("Form", "Shift"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())

